
#ifndef _XSECURITY_H_
#define _XSECURITY_H_

/*
 * Copyright (c) SecureWare, Inc, 1994, All rights reserved.
 *
 * @(#)01 1.6 x11r5/extensions/include/xsecurity/XSecurity.h, xsecext, sco_3.0, 301FCS 10:45:33 1/19/95, SecureWare, Inc.
 */

#if SEC_BASE

#include <X11/extensions/Xsecdefs.h>

extern XExtCodes **_XSecCodes;

#define XSecExtCodes(dpy) _XSecCodes[ConnectionNumber(dpy)]

#if defined(__STDC__) && !defined(UNIXCPP)
#define GetExtReq(name, req, dpy) \
        WORD64ALIGN\
	if ((dpy->bufptr + SIZEOF(x##name##Req)) > dpy->bufmax)\
	    _XFlush(dpy);\
	req = (x##name##Req *)(dpy->last_req = dpy->bufptr);\
	req->reqType =  XSecExtCodes(dpy)->major_opcode;\
	req->secReqType = X_##name;\
	req->length = (SIZEOF(x##name##Req))>>2;\
	dpy->bufptr += SIZEOF(x##name##Req);\
	dpy->request++
#define GetEmptyExtReq(name, req, dpy) \
        WORD64ALIGN\
	if ((dpy->bufptr + SIZEOF(xReq)) > dpy->bufmax)\
	    _XFlush(dpy);\
	req = (xReq *) (dpy->last_req = dpy->bufptr);\
	req->reqType = XSecExtCodes(dpy)->major_opcode;\
	req->data = X_##name;\
	req->length = 1;\
	dpy->bufptr += SIZEOF(xReq);\
	dpy->request++
#else
#define GetExtReq(name, req, dpy) \
        WORD64ALIGN\
	if ((dpy->bufptr + SIZEOF(x/**/name/**/Req)) > dpy->bufmax)\
	    _XFlush(dpy);\
	req = (x/**/name/**/Req *)(dpy->last_req = dpy->bufptr);\
	req->reqType =  XSecExtCodes(dpy)->major_opcode;\
	req->secReqType = X_/**/name;\
	req->length = (SIZEOF(x/**/name/**/Req))>>2;\
	dpy->bufptr += SIZEOF(x/**/name/**/Req);\
	dpy->request++
#define GetEmptyExtReq(name, req, dpy) \
        WORD64ALIGN\
	if ((dpy->bufptr + SIZEOF(xReq)) > dpy->bufmax)\
	    _XFlush(dpy);\
	req = (xReq *) (dpy->last_req = dpy->bufptr);\
	req->reqType = XSecExtCodes(dpy)->major_opcode;\
	req->data = X_/**/name;\
	req->length = 1;\
	dpy->bufptr += SIZEOF(xReq);\
	dpy->request++
#endif

/*
 * Event number functions (pass (Display *) as argument).
 */
extern int	XSecurityEventBase(Display *);

#define	InactivityTimeoutEventType(dpy)		(XSecurityEventBase(dpy) + \
						    InactivityTimeoutEvent)
#define	SelectionRedirectEventType(dpy)		(XSecurityEventBase(dpy) + \
						    SelectionRedirectEvent)
#define	SelectionForwardNotifyEventType(dpy)	(XSecurityEventBase(dpy) + \
						    SelectionForwardNotifyEvent)
#define	ServiceNotifyEventType(dpy)		(XSecurityEventBase(dpy) + \
						    ServiceNotifyEvent)
#define	AttributeNotifyEventType(dpy)		(XSecurityEventBase(dpy) + \
						    AttributeNotifyEvent)
#define	GrabOverrideNotifyEventType(dpy)	(XSecurityEventBase(dpy) + \
						    GrabOverrideNotifyEvent)
#if SEC_ILB
#define ClearResetNotifyEventType(dpy)		(XSecurityEventBase(dpy) + \
						    ClearResetNotifyEvent)
#endif
#define ScreenNotifyEventType(dpy)		(XSecurityEventBase(dpy) + \
						    ScreenNotifyEvent)

typedef struct _XSvcReg {
    Atom	service_name;
    Window	service_window;
} XServiceRegistration;

typedef unsigned int XPrivilege;

typedef struct _XPrivSet {
    PrivWord	priv[X_PRIVSET_SIZE];
} XPrivilegeSet;

typedef struct _XPerms {
    Atom	object_type;
    unsigned long perms;
} XPermissions;

typedef struct _XAttr {
    Atom	type;
    int		length;
    void	*attribute;
} XAttribute;

/* Access control list definitions */

#define	WILDCARD_HOST	-1		/* any host (family) */
#define WILDCARD_USER	-1		/* any user (user) */

#define ACCESS_DENIED	0		/* deny access (access) */
#define ACCESS_GRANTED	1		/* permit access (access) */

typedef struct _XDisplayACL {
    int		family;		/* address family, FamilyInternet, ... */
    int 	length;		/* length of network address */
    char 	*address;	/* non-null terminated network address */
    int 	user;		/* user ID or Wildcard user */
    int 	access;		/* {ACCESS_DENIED, ACCESS_GRANTED} */
} XDisplayACL;

/*
 * This structure is hung from each display connection.  It contains
 * display-specific variables.  No one outside of the library should
 * be referencing it unless there is macro provided.
 */
typedef struct _XSecExtInfo {
    Atom	secatoms[21];		/* Security atoms */
    int		privs_initd;
    XPrivilegeSet effprivs;		/* For privilege module */
    void	*data_ptr;		/* Used by RegisterService */
    int		screen_height;		/* Used by AllocSecurityStripe */
    int		screen_mheight;		/* Used by AllocSecurityStripe */
    int		allocd_stripe;		/* Used by AllocSecurityStripe */
} XSecExtInfo;

#define XEHeadOfExtensionList(obj)	((XExtData **)obj)

#define XSecExtInfoFromDisplay(dpy) \
	((XSecExtInfo *) (XFindOnExtensionList(XEHeadOfExtensionList(dpy), \
	 XSecExtCodes(dpy)->extension)->private_data))

#define XA_SEC_WINDOW(dpy) 	(XSecExtInfoFromDisplay(dpy)->secatoms[ 0])
#define XA_SEC_PIXMAP(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 1])
#define XA_SEC_GCONTEXT(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 2])
#define XA_SEC_FONT(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 3])
#define XA_SEC_CURSOR(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 4])
#define XA_SEC_PROPERTY(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 5])
#define XA_SEC_ATOM(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 6])
#define XA_SEC_COLORMAP(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 7])
#define XA_SEC_COLOR_CELL(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[ 8])
#define XA_SEC_CLIENT_LUID(dpy) (XSecExtInfoFromDisplay(dpy)->secatoms[ 9])
#define XA_SEC_CLIENT_UID(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[10])
#define XA_SEC_CLIENT_GID(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[11])
#define XA_SEC_MAX_PRIVILEGES(dpy) (XSecExtInfoFromDisplay(dpy)->secatoms[12])
#define XA_SEC_EFF_PRIVILEGES(dpy) (XSecExtInfoFromDisplay(dpy)->secatoms[13])
#define XA_SEC_PROCESS_ID(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[14])
#if SEC_MAC
#define XA_SEC_MAC_LABEL(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[15])
#endif
#if SEC_ILB
#define XA_SEC_INFO_LABEL(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[16])
#define XA_SEC_INPUT_INFO_LABEL(dpy) (XSecExtInfoFromDisplay(dpy)->secatoms[17])
#endif
#define XA_SEC_PERMISSIONS(dpy)	(XSecExtInfoFromDisplay(dpy)->secatoms[18])
#define XA_SEC_UID(dpy)		(XSecExtInfoFromDisplay(dpy)->secatoms[19])
#define XA_SEC_GID(dpy)		(XSecExtInfoFromDisplay(dpy)->secatoms[20])

typedef struct _XSecAnyEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	window;		/* Affected window */
} XSecAnyEvent;

typedef struct _XAttrEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	window;		/* Affected window */
	Atom	attribute_type;	/* Which attribute was changed */
} XAttributeChangeEvent;

typedef struct _XGrOvEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	window;		/* Affected window */
	int	grab_type;	/* Which grab was overridden */
} XGrabOverrideEvent;

typedef struct _XSelFwdEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	redirect_window; /* Window to which the event is reported */
	Window	sel_window;	/* Window to which request was forwarded */
	Atom	selection;	/* Identifies selection */
	Time	time;		/* Time from ConvertSelection req */
	XID	sel_client;	/* Selection owner client bits */
} XSelectionForwardEvent;

typedef struct _XSelRedirEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	redirect_window; /* Window to which event is reported */
	Window	requestor;	/* Requester window */
	Atom	selection;	/* Identifies selection */
	Atom	target;		/* Desired data type */
	Atom	property;	/* Where converted data should be stored */
	Time	time;		/* Time from ConvertSelection req */
	XID	req_client;	/* Requesting client */
} XSelectionRedirectEvent;

typedef struct _XSvcEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	registration_window; /* Registration window */
	Atom	service_name;	/* Which service is affected */
	int	detail;		/* What happened */
} XServiceEvent;

typedef struct _XTimeoutEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	window;		/* Root Window */
} XInactivityTimeoutEvent;

#if SEC_ILB

typedef struct _XClearResetNotifyEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	window;		/* Target window */
} XClearResetNotifyEvent;

#endif

typedef struct _XScreenNotifyEvent {
	int 	type;
	unsigned long serial;   /* # of last request processed by server */
	Bool	send_event;	/* true if this came from a SendEvent request */
	Display	*display;	/* Display the event was read from */
	Window	window;		/* Root Window */
	Bool	acquired;	/* Screen acquired? */
} XScreenNotifyEvent;

typedef union _XSecEvent {
    int				type;
    XSecAnyEvent		xany;
    XAttributeChangeEvent	xattr;
    XGrabOverrideEvent		xgrabov;
    XSelectionForwardEvent	xselfwd;
    XSelectionRedirectEvent	xselrdr;
    XServiceEvent		xsvc;
    XInactivityTimeoutEvent	xto;
#if SEC_ILB
    XClearResetNotifyEvent	xclrreset;
#endif
    XScreenNotifyEvent		xscreen;
    XEvent			xcore;		/* Ensure correct size */
} XSecEvent;

/* 
 * Macros.
 */

#if SEC_MAC
#define XAllocMACLabel()	((MACLabel *)malloc(MAC_LABEL_SIZE_RND))
#define XFreeMACLabel(l)	free((char *)l)
#define XCopyMACLabel(src,dest)	memcpy(dest, src, MAC_LABEL_SIZE);
#endif
#if SEC_ILB
#define XAllocInfoLabel()	((InfoLabel *)malloc(INFO_LABEL_SIZE_RND))
#define XFreeInfoLabel(l)	free((char *)l)
#define XCopyInfoLabel(src,dest)	memcpy(dest, src, INFO_LABEL_SIZE);
#endif

#define XPrivilegeInSet(set, p)	PrivIsInSet((set)->priv, p)

/*
 * Function Prototypes
 */

extern Status		XSecurityExtensionInit(Display *);
extern Status		XAllocReservedColor(Display *, Colormap, XColor *);
extern void		XAllocSecurityStripe(Display *, Screen *, Window, int);
extern void		XAllowUntrustedClients(Display *);
extern void		XSelectSecurityEvents(Display *, Window, unsigned long);
extern void		XCopyProperty(Display *, Window, Window, Atom, Atom,
			    int, Bool);
extern void		XKillAllClients(Display *);
extern Status		XGetInactivityTimeout(Display *, int *, int *);
extern void		XSetInactivityTimeout(Display *, int);
extern void		XSetScreenLock(Display *, Bool);
extern void		XListenToTrustedClient(Display *, Bool);
extern Status		XSwitchScreen(Display *, int, int *);
#if SEC_MAC
extern void		XSetSessionClearance(Display *, MACLabel *, MACLabel *);
#endif

extern Status		XGetClientAttributes(Display *, XID, Atom *, int,
			    XAttribute **, int *);
extern Status		XGetAllClientAttributes(Display *, XID, XAttribute **,
			    int *);
extern Status		XGetClientLUID(Display *, XID, LoginUserID *);
extern Status		XGetClientUID(Display *, XID, UserID *);
extern Status		XGetClientGIDs(Display *, XID, GroupID **, int *);
#if SEC_MAC
extern Status		XGetClientMACLabel(Display *, XID, MACLabel **);
#if SEC_ILB
extern Status		XGetClientInfoLabel(Display *, XID, InfoLabel **);
#endif
#endif
extern Status		XGetClientMaxPrivs(Display *, XID, XPrivilegeSet *);
extern Status		XGetClientEffPrivs(Display *, XID, XPrivilegeSet *);
extern Status		XGetClientProcessID(Display *, XID, ProcessID *);
extern void		XFreeAttributes(XAttribute *, int);

extern XPrivilegeSet *	XSetEffectivePrivileges(Display *, XPrivilegeSet *);
extern XPrivilegeSet *	XRaisePrivileges(Display *, XPrivilegeSet *);
extern XPrivilegeSet *	XDropPrivileges(Display *, XPrivilegeSet *);
extern XPrivilegeSet *	XBuildPrivilegeSet(XPrivilegeSet *, ...);
extern XPrivilegeSet *	XAugmentPrivilegeSet(XPrivilegeSet *, ...);

extern Status		XRegisterService(Display *, Window, Atom, Window, Bool);
extern void		XUnregisterService(Display *, Window, Atom);
extern
XServiceRegistration *	XListServices(Display *, Window, int *);
extern Window		XGetServiceWindow(Display *, Window, Atom);

extern Status		XGetObjectAttributes(Display *, XID, XID, XID,
			    Atom, Atom *, int, XAttribute **, int *);
extern Status		XGetAllObjectAttributes(Display *, XID, XID, XID,
			    Atom, XAttribute **, int *);
extern Status		XGetObjectUID(Display *, XID, XID, XID, Atom,
			    UserID *);
extern Status		XGetObjectGID(Display *, XID, XID, XID, Atom,
			    GroupID *);
extern Status		XGetObjectPermissions(Display *, XID, XID, XID,
			    Atom, unsigned long *);
#if SEC_MAC
extern Status		XGetObjectMACLabel(Display *, XID, XID, XID,
			    Atom, MACLabel **);
#if SEC_ILB
extern Status		XGetObjectInfoLabel(Display *, XID, XID, XID,
			    Atom, InfoLabel **);
extern Status		XGetObjectInputInfoLabel(Display *, XID, XID, XID,
			    Atom, InfoLabel **);
#endif
#endif
extern void		XSetObjectAttributes(Display *, XID, XID, XID,
			    Atom, XAttribute *, int);
extern void		XSetObjectUID(Display *, XID, XID, XID, Atom, UserID);
extern void		XSetObjectGID(Display *, XID, XID, XID, Atom, GroupID);
extern void		XSetObjectPermissions(Display *, XID, XID, XID, Atom,
			    unsigned long);
#if SEC_MAC
extern void		XSetObjectMACLabel(Display *, XID, XID, XID, Atom,
			    MACLabel *);
#if SEC_ILB
extern void		XSetObjectInfoLabel(Display *, XID, XID, XID, Atom,
			    InfoLabel *);
extern void		XSetObjectInputInfoLabel(Display *, XID, XID, XID,
			    Atom, InfoLabel *);
#endif

extern Atom		XComposePropertyAtom(Display *, Atom, MACLabel *);
extern Atom		XDecomposePropertyAtom(Display *, Atom, MACLabel **);
extern void		XSetMultilevelPropertyMode(Display *, int, int,
			    MACLabel *);
extern Status		XGetMultilevelPropertyMode(Display *, int *, int *,
			    MACLabel **);
#endif

extern void		XAllowGrabs(Display *);
extern void		XRestrictGrabs(Display *);
extern void		XOverrideGrabButton(Display *, unsigned int,
			    unsigned int, Window, Bool, unsigned int, int,
			    int, Window, Cursor);
extern void		XOverrideGrabKey(Display *, int, unsigned int,
			    Window, Bool, int, int);
extern int		XOverrideGrabKeyboard(Display *, Window, Bool, int,
			    int, Time);
extern int		XOverrideGrabPointer(Display *, Window, Bool,
			    unsigned int, int, int, Window, Cursor, Time);

Status			XGetAllDefaultPermissions(Display *, int,
			    XPermissions **, int *);
unsigned long		XGetDefaultPermissions(Display *, int, Atom);
void			XSetDefaultPermissions(Display *, int, XPermissions *,
			    int);

extern Status		XGetDisplayACL(Display *, XDisplayACL ***,
			    int *, int *);
extern void		XSetDisplayACL(Display *, XDisplayACL **, int);

#if SEC_ILB
extern void		XClearResetWindow(Display *, Window);
#endif

#endif /* SEC_BASE */
#endif /* _XSECURITY_H_ */

